@@ -4,17 +4,17 @@
 from __future__ import annotations
 
 import logging
+import os
 import os.path
-import sys
+import pathlib
 from io import BytesIO, FileIO
 from shutil import copy
-from time import sleep
 
-import psutil
 import pytest
 
 import pikepdf
 from pikepdf import Pdf, PdfError
+from pikepdf._io import atomic_overwrite
 
 # pylint: disable=redefined-outer-name
 
@@ -62,7 +62,7 @@ def test_fail_only_overwrite_input_check(monkeypatch, resources, outdir):
         def mockraise(*args):
             raise OSError("samefile mocked")
 
-        monkeypatch.setattr(os.path, 'samefile', mockraise)
+        monkeypatch.setattr(pathlib.Path, 'samefile', mockraise)
         with pytest.raises(OSError, match=r'samefile mocked'):
             p.save(outdir / 'wouldwork.pdf')
 
@@ -101,104 +101,6 @@ def test_invalid_output_stream(sandwich, bio_class, exc_type):
         sandwich.save(bio, static_id=True)
 
 
-def _file_descriptor_is_open(
-    path, retry_until: bool, retries: int = 3, delay: float = 1.0
-):
-    process = psutil.Process()
-    for _ in range(retries):
-        is_open = any((f.path == str(path.resolve())) for f in process.open_files())
-        if is_open == retry_until:
-            return is_open
-        sleep(delay)
-
-    return is_open
-
-
-def _skip_file_descriptor_checks_if_not_supported():
-    if sys.platform == 'win32' or sys.platform.startswith('freebsd'):
-        pytest.skip(
-            "psutil documentation warns that .open_files() has problems on these"
-        )
-    elif sys.implementation.name == 'pypy':
-        pytest.skip("randomly fails on pypy")
-
-
-@pytest.fixture
-def file_descriptor_is_open():
-    _skip_file_descriptor_checks_if_not_supported()
-
-    def _wait_till_open(path):
-        return _file_descriptor_is_open(path, retry_until=True)
-
-    return _wait_till_open
-
-
-@pytest.fixture
-def file_descriptor_is_closed():
-    _skip_file_descriptor_checks_if_not_supported()
-
-    def _wait_till_closed(path):
-        return not _file_descriptor_is_open(path, retry_until=False)
-
-    return _wait_till_closed
-
-
-def test_open_named_file_closed(
-    resources, file_descriptor_is_open, file_descriptor_is_closed
-):
-    path = resources / 'pal.pdf'
-    pdf = Pdf.open(path)  # no with clause
-    assert file_descriptor_is_open(path)
-
-    pdf.close()
-    assert file_descriptor_is_closed(path), "pikepdf did not close a stream it opened"
-
-
-def test_streamed_file_not_closed(resources, file_descriptor_is_open):
-    path = resources / 'pal.pdf'
-    stream = path.open('rb')
-    pdf = Pdf.open(stream)  # no with clause
-    assert file_descriptor_is_open(path)
-
-    pdf.close()
-    assert file_descriptor_is_open(path), "pikepdf closed a stream it did not open"
-
-
-@pytest.mark.parametrize('branch', ['success', 'failure'])
-def test_save_named_file_closed(
-    resources, outdir, file_descriptor_is_open, file_descriptor_is_closed, branch
-):
-    with Pdf.open(resources / 'pal.pdf') as pdf:
-        path = outdir / "pal.pdf"
-
-        def confirm_opened(progress_percent):
-            if progress_percent == 0:
-                assert file_descriptor_is_open(path)
-            if progress_percent > 0 and branch == 'failure':
-                raise ValueError('failure branch')
-
-        try:
-            pdf.save(path, progress=confirm_opened)
-        except ValueError:
-            pass
-        assert file_descriptor_is_closed(
-            path
-        ), "pikepdf did not close a stream it opened"
-
-
-def test_save_streamed_file_not_closed(resources, outdir, file_descriptor_is_open):
-    with Pdf.open(resources / 'pal.pdf') as pdf:
-        path = outdir / "pal.pdf"
-        stream = path.open('wb')
-
-        def confirm_opened(progress_percent):
-            if progress_percent == 0:
-                assert file_descriptor_is_open(path)
-
-        pdf.save(stream, progress=confirm_opened)
-        assert file_descriptor_is_open(path), "pikepdf closed a stream it did not open"
-
-
 class ExpectedError(Exception):
     pass
 
@@ -210,10 +112,10 @@ def test_file_without_fileno(resources):
 
     f = FileWithoutFileNo(resources / 'pal.pdf', 'rb')
     with pytest.raises(ExpectedError):
-        Pdf.open(f, access_mode=pikepdf._qpdf.AccessMode.mmap_only)
+        Pdf.open(f, access_mode=pikepdf._core.AccessMode.mmap_only)
 
     # Confirm we automatically fallback to stream
-    with Pdf.open(f, access_mode=pikepdf._qpdf.AccessMode.mmap) as pdf:
+    with Pdf.open(f, access_mode=pikepdf._core.AccessMode.mmap) as pdf:
         assert pdf.filename
 
 
@@ -225,10 +127,10 @@ def test_file_deny_mmap(resources, monkeypatch):
 
     monkeypatch.setattr(mmap, 'mmap', raises_oserror)
     with pytest.raises(OSError):
-        Pdf.open(resources / 'pal.pdf', access_mode=pikepdf._qpdf.AccessMode.mmap_only)
+        Pdf.open(resources / 'pal.pdf', access_mode=pikepdf._core.AccessMode.mmap_only)
 
     with Pdf.open(
-        resources / 'pal.pdf', access_mode=pikepdf._qpdf.AccessMode.default
+        resources / 'pal.pdf', access_mode=pikepdf._core.AccessMode.default
     ) as pdf:
         assert len(pdf.pages) == 1
 
@@ -242,7 +144,7 @@ def test_mmap_only_file(resources):
 
     f = UnreadableFile(resources / 'pal.pdf', 'rb')
     with pytest.raises(ExpectedError):
-        Pdf.open(f, access_mode=pikepdf._qpdf.AccessMode.stream)
+        Pdf.open(f, access_mode=pikepdf._core.AccessMode.stream)
 
 
 def test_save_bytesio(resources, outpdf):
@@ -295,7 +197,55 @@ def test_read_after_close(resources):
 
 def test_logging(caplog):
     caplog.set_level(logging.INFO)
-    pikepdf._qpdf._log_info("test log message")
-    assert [("pikepdf._qpdf", logging.INFO)] == [
+    pikepdf._core._test.log_info("test log message")
+    assert [("pikepdf._core", logging.INFO)] == [
         (rec[0], rec[1]) for rec in caplog.record_tuples
     ]
+
+
+def test_atomic_overwrite_new(tmp_path):
+    new_file = tmp_path / 'new.pdf'
+    assert not new_file.exists()
+
+    with pytest.raises(ValueError, match='oops'), atomic_overwrite(new_file) as f:
+        f.write(b'a failed write should not produce an invalid file')
+        raise ValueError('oops')
+    assert not new_file.exists()
+
+    assert list(tmp_path.glob('*.pikepdf')) == [], "Temporary files were not cleaned up"
+
+
+def test_atomic_overwrite_existing(tmp_path):
+    existing_file = tmp_path / 'existing.pdf'
+    existing_file.write_bytes(b'existing')
+
+    with atomic_overwrite(existing_file) as f:
+        f.write(b'new')
+    assert existing_file.read_bytes() == b'new'
+
+    with pytest.raises(ValueError, match='oops'), atomic_overwrite(existing_file) as f:
+        f.write(b'a failed update should not corrupt the file')
+        raise ValueError('oops')
+    assert existing_file.read_bytes() == b'new'
+
+    assert list(tmp_path.glob('*.pikepdf')) == [], "Temporary files were not cleaned up"
+
+
+def test_memory_to_path(resources, tmp_path):
+    bio = BytesIO((resources / 'sandwich.pdf').read_bytes())
+    with Pdf.open(bio) as pdf:
+        assert len(pdf.pages) == 1
+        pdf.save(str(tmp_path / 'out.pdf'))
+
+
+def test_newline_handling(resources):
+    with Pdf.open(
+        resources / 'newline-buffer-test.pdf',
+        access_mode=pikepdf._core.AccessMode.mmap_only,
+    ) as pdf:
+        assert pdf.check() == []
+    with Pdf.open(
+        resources / 'newline-buffer-test.pdf',
+        access_mode=pikepdf._core.AccessMode.stream,
+    ) as pdf:
+        assert pdf.check() == []
