@@ -35,12 +35,12 @@ from pikepdf import (
     Stream,
     String,
 )
-from pikepdf import _qpdf as qpdf
+from pikepdf import _core as core
 from pikepdf.models import parse_content_stream
 
 # pylint: disable=eval-used, redefined-outer-name
 
-encode = qpdf._encode
+encode = core._encode
 
 
 def test_none():
@@ -112,23 +112,27 @@ def test_decimal_from_float(f):
         except InvalidOperation:
             return  # PDF doesn't support exponential notation
         try:
-            py_d = Object.parse(decstr)
+            py_d = Object.parse(decstr.encode('pdfdoc'))
         except RuntimeError as e:
             if 'overflow' in str(e) or 'underflow' in str(e):
-                py_d = Object.parse(str(f))
+                py_d = Object.parse(f.encode('pdfdoc'))
 
         assert isclose(py_d, d, abs_tol=1e-5), (d, f.hex())
     else:
-        with pytest.raises(PdfError):
+        with pytest.raises(PdfError), pytest.deprecated_call():
             Object.parse(str(d))
 
 
 def test_qpdf_real_to_decimal():
-    assert isclose(qpdf._new_real(1.2345, 4), Decimal('1.2345'), abs_tol=1e-5)
-    assert isclose(qpdf._new_real('2.3456'), Decimal('2.3456'), abs_tol=1e-5)
+    assert isclose(core._new_real(1.2345, 4), Decimal('1.2345'), abs_tol=1e-5)
+    assert isclose(core._new_real('2.3456'), Decimal('2.3456'), abs_tol=1e-5)
 
 
 @skip_if_pypy
+@pytest.mark.xfail(
+    sys.version_info >= (3, 12),
+    reason="https://github.com/python/cpython/issues/105003",
+)
 def test_stack_depth():
     a = [42]
     for _ in range(100):
@@ -254,6 +258,10 @@ class TestArray:
         assert pikepdf.Array([0, 1, 2, 3]).is_rectangle
         assert not pikepdf.Array(['a', '2', 3, 4]).is_rectangle
 
+    def test_array_bool(self):
+        assert bool(pikepdf.Array([1, 2, 3])) is True
+        assert bool(pikepdf.Array([])) is False
+
 
 def test_no_len():
     with pytest.raises(TypeError):
@@ -299,6 +307,14 @@ class TestName:
         foo = Name('/Foo')
         assert Name(foo) == foo
 
+    def test_name_bool(self):
+        assert bool(Name('/Foo')) is True
+        # Currently we forbid the empty name. All creatable names are true.
+        with pytest.raises(ValueError):
+            bool(Name('/'))
+        with pytest.raises(ValueError):
+            bool(Name(''))
+
 
 class TestHashViolation:
     def check(self, a, b):
@@ -309,11 +325,11 @@ class TestHashViolation:
         assert Name('/Foo') != String('/Foo')
 
     def test_numbers(self):
-        self.check(Object.parse('1.0'), 1)
-        self.check(Object.parse('42'), 42)
+        self.check(Object.parse(b'1.0'), 1)
+        self.check(Object.parse(b'42'), 42)
 
     def test_bool_comparison(self):
-        self.check(Object.parse('0.0'), False)
+        self.check(Object.parse(b'0.0'), False)
         self.check(True, 1)
 
     def test_string(self):
@@ -336,76 +352,6 @@ def test_not_constructible():
         Object()
 
 
-class TestRepr:
-    def test_repr_dict(self):
-        d = Dictionary(
-            {
-                '/Boolean': True,
-                '/Integer': 42,
-                '/Real': Decimal('42.42'),
-                '/String': String('hi'),
-                '/Array': Array([1, 2, 3.14]),
-                '/Operator': Operator('q'),
-                '/Dictionary': Dictionary({'/Color': 'Red'}),
-                '/None': None,
-            }
-        )
-        short_pi = '3.14'
-        expected = (
-            """\
-            pikepdf.Dictionary({
-                "/Array": [ 1, 2, Decimal('%s') ],
-                "/Boolean": True,
-                "/Dictionary": {
-                    "/Color": "Red"
-                },
-                "/Integer": 42,
-                "/None": None,
-                "/Operator": pikepdf.Operator("q"),
-                "/Real": Decimal('42.42'),
-                "/String": "hi"
-            })
-        """
-            % short_pi
-        )
-
-        def strip_all_whitespace(s):
-            return ''.join(s.split())
-
-        assert strip_all_whitespace(repr(d)) == strip_all_whitespace(expected)
-        assert eval(repr(d)) == d
-
-    def test_repr_scalar(self):
-        scalars = [
-            False,
-            666,
-            Decimal('3.14'),
-            String('scalar'),
-            Name('/Bob'),
-            Operator('Q'),
-        ]
-        for s in scalars:
-            assert eval(repr(s)) == s
-
-    def test_repr_indirect(self, resources):
-        with pikepdf.open(resources / 'graph.pdf') as graph:
-            repr_page0 = repr(graph.pages[0])
-            assert repr_page0[0] == '<', 'should not be constructible'
-
-    def test_repr_circular(self):
-        with pikepdf.new() as pdf:
-            pdf.Root.Circular = pdf.make_indirect(Dictionary())
-            pdf.Root.Circular.Parent = pdf.make_indirect(Dictionary())
-            pdf.Root.Circular.Parent = pdf.make_indirect(pdf.Root.Circular)
-            assert '.get_object' in repr(pdf.Root.Circular)
-
-    def test_repr_indirect_page(self, resources):
-        with pikepdf.open(resources / 'outlines.pdf') as outlines:
-            assert 'from_objgen' in repr(outlines.Root.Pages.Kids)
-            # An indirect page reference in the Dests name tree
-            assert 'from_objgen' in repr(outlines.Root.Names.Dests.Kids[0].Names[1])
-
-
 def test_operator_inline(resources):
     with pikepdf.open(resources / 'image-mono-inline.pdf') as pdf:
         instructions = parse_content_stream(pdf.pages[0], operators='BI ID EI')
@@ -453,9 +399,8 @@ class TestDictionary:
             pass
 
     def test_str(self):
-        d = pikepdf.Dictionary(A='a')
-        with pytest.raises(NotImplementedError):
-            str(d)
+        d = pikepdf.Dictionary(ABCD='abcd')
+        assert 'ABCD' in str(d)
 
     def test_attr(self):
         d = pikepdf.Dictionary(A='a')
@@ -506,6 +451,10 @@ class TestDictionary:
         assert d == d2
         assert d is not d2
 
+    def test_dict_bool(self):
+        assert bool(pikepdf.Dictionary()) is False
+        assert bool(pikepdf.Dictionary(One=1)) is True
+
 
 def test_not_convertible():
     class PurePythonObj:
@@ -551,10 +500,10 @@ def test_json():
 class TestStream:
     @pytest.fixture(scope="function")
     def abcxyz_stream(self):
-        pdf = pikepdf.new()
-        data = b'abcxyz'
-        stream = Stream(pdf, data)
-        return stream
+        with pikepdf.new() as pdf:
+            data = b'abcxyz'
+            stream = Stream(pdf, data)
+            yield stream
 
     def test_stream_isinstance(self):
         pdf = pikepdf.new()
@@ -652,6 +601,12 @@ class TestStream:
         with pikepdf.open(outpdf) as pdf2:
             assert pdf2.Root.SomeStream.read_bytes() == b'blahblah'
 
+    def test_stream_bool(self):
+        pdf = pikepdf.new()
+        assert bool(Stream(pdf, b'')) is False
+        stream = Stream(pdf, b'blahblah')
+        assert bool(stream) is True
+
 
 @pytest.fixture
 def sandwich(resources):
@@ -662,8 +617,8 @@ def sandwich(resources):
 class TestStreamReadWrite:
     @pytest.fixture
     def stream_object(self):
-        pdf = pikepdf.new()
-        return Stream(pdf, b'abc123xyz')
+        with pikepdf.new() as pdf:
+            yield Stream(pdf, b'abc123xyz')
 
     def test_basic(self, stream_object):
         stream_object.write(b'abc')
@@ -801,12 +756,18 @@ class TestOperator:
         ):
             Operator('Do')['x'] = 42
 
+    def test_operator_bool(self):
+        assert bool(Operator('Do')) is True
+        assert bool(Operator('')) is False
+
 
 def test_object_mapping(sandwich):
     object_mapping = sandwich.pages[0].images
     assert '42' not in object_mapping
     assert '/R12' in object_mapping
     assert '/R12' in object_mapping.keys()
+    assert object_mapping.get(Name.R12) == object_mapping[Name.R12]
+    assert object_mapping.get(Name.DoesNotExist, 42) == 42
 
 
 def test_replace_object(sandwich):
@@ -822,3 +783,9 @@ def test_swap_object(resources):
         pdf._swap_objects(pdf.pages[0].objgen, pdf.pages[1].objgen)
         assert pdf.pages[1].MarkPage0
         assert Name.MarkPage0 not in pdf.pages[0]
+
+
+class TestString:
+    def test_string_bool(self):
+        assert bool(String('')) is False
+        assert bool(String('abc')) is True
